@@ -2,7 +2,7 @@ import abc
 from typing import Callable, Optional, Tuple
 
 import numpy as np
-from yupi import Trajectory
+from yupi import Trajectory, VelocityMethod, WindowType
 
 
 class Generator(metaclass=abc.ABCMeta):
@@ -172,6 +172,10 @@ class RandomWalkGenerator(Generator):
                     dt=self.dt,
                     t=self.t,
                     traj_id=f"{self.traj_id} {i + 1}",
+                    vel_est={
+                        "method": VelocityMethod.LINEAR_DIFF,
+                        "window_type": WindowType.FORWARD,
+                    },
                 )
             )
         return trajs
@@ -184,7 +188,7 @@ class _LangevinGenerator(Generator):
         dim: int = 1,
         N: int = 1,
         dt: float = 1.0,
-        tau: float = 1.0,
+        gamma: float = 1.0,
         sigma: float = 1.0,
         v0: np.ndarray = None,
         r0: np.ndarray = None,
@@ -197,7 +201,7 @@ class _LangevinGenerator(Generator):
         self.traj_id = "Langevin"
 
         # Model parameters
-        self.tau = tau  # Relaxation time
+        self.gamma = gamma  # Relaxation time
         self.sigma = sigma  # Noise scale parameter
 
         # Initial conditions
@@ -212,7 +216,7 @@ class _LangevinGenerator(Generator):
 
     # Intrinsic reference parameters
     def _set_scaling_params(self):
-        self.t_scale = self.tau  # Time scale
+        self.t_scale = self.gamma ** -1  # Time scale
         self.v_scale = self.sigma * np.sqrt(self.t_scale)  # Speed scale
         self.r_scale = self.v_scale * self.t_scale  # Length scale
 
@@ -232,7 +236,7 @@ class _LangevinGenerator(Generator):
         # Initial positions
         if self.r0 is None:
             self.r[0] = np.zeros((self.dim, self.N))  # Default
-        elif np.shape(self.r0) == (self.dim, self.N) or np.shape(self.r0) == ():
+        elif np.shape(self.r0) == (self.dim, self.N) or np.ndim(self.r0) == 0:
             self.r[0] = self.r0  # User input
         else:
             raise ValueError(
@@ -244,7 +248,7 @@ class _LangevinGenerator(Generator):
         # Initial velocities
         if self.v0 is None:
             self.v[0] = self.rng.normal(size=(self.dim, self.N))  # Default
-        elif np.shape(self.v0) == (self.dim, self.N) or np.shape(self.v0) == ():
+        elif np.shape(self.v0) == (self.dim, self.N) or np.ndim(self.v0) == 0:
             self.v[0] = self.v0  # User input
         else:
             raise ValueError(
@@ -260,14 +264,13 @@ class _LangevinGenerator(Generator):
     # Solve dimensionless Langevin Equation using
     # the numerical method of Euler-Maruyama
     def _solve(self):
+        sqrt_dt = np.sqrt(self.dt)
         for i in range(self.n - 1):
             # Solving for position
             self.r[i + 1] = self.r[i] + self.v[i] * self.dt
 
             # Solving for velocity
-            self.v[i + 1] = (
-                self.v[i] + -self.v[i] * self.dt + np.sqrt(self.dt) * self.noise[i]
-            )
+            self.v[i + 1] = self.v[i] - self.v[i] * self.dt + self.noise[i] * sqrt_dt
 
     # Scale by intrinsic reference quantities
     def _set_scale(self):
@@ -310,8 +313,8 @@ class LangevinGenerator(_LangevinGenerator):
         Number of simulated trajectories, by default 1.
     dt : float, optional
         Time step, by default 1.0.
-    tau : float, optional
-        Persistence time, by default 1.
+    gamma : float, optional
+        Drag parameter or inverse of the persistence time, by default 1.
     sigma : float, optional
         Noise intensity (i.e., scale parameter of noise pdf), by default 1.
     bounds: np.ndarray, optional
@@ -334,7 +337,7 @@ class LangevinGenerator(_LangevinGenerator):
         dim: int = 1,
         N: int = 1,
         dt: float = 1.0,
-        tau: float = 1.0,
+        gamma: float = 1.0,
         sigma: float = 1.0,
         bounds: np.ndarray = None,
         bounds_extent: np.ndarray = None,
@@ -344,7 +347,7 @@ class LangevinGenerator(_LangevinGenerator):
         seed: Optional[int] = None,
     ):
 
-        super().__init__(T, dim, N, dt, tau, sigma, v0, r0, seed)
+        super().__init__(T, dim, N, dt, gamma, sigma, v0, r0, seed)
 
         # Verify if there is any boundary
         self.bounds = bounds
@@ -445,6 +448,7 @@ class LangevinGenerator(_LangevinGenerator):
     # Solve dimensionless Langevin Equation using
     # the numerical method of Euler-Maruyama
     def _solve(self):
+        sqrt_dt = np.sqrt(self.dt)
         for i in range(self.n - 1):
             # Solving for position
             self.r[i + 1] = self.r[i] + self.v[i] * self.dt
@@ -452,8 +456,8 @@ class LangevinGenerator(_LangevinGenerator):
             # Solving for velocity
             self.v[i + 1] = (
                 self.v[i]
-                + -self.v[i] * self.dt
-                + np.sqrt(self.dt) * self.noise[i]
+                - self.v[i] * self.dt
+                + self.noise[i] * sqrt_dt
                 + self._bound_force(self.r[i]) * self.dt
             )
 
@@ -494,10 +498,8 @@ class _DiffDiffGenerator(Generator):
         self.t = np.arange(self.n, dtype=np.float32)  # Time array
         self.r = np.empty(self.shape)  # Position array
         self.Y = np.empty((dim_aux, N))  # Aux variable: square of diffusivity
-        self.noise_r = None  # Noise array for position (filled in _set_noise method)
-        self.noise_Y = (
-            None  # Noise array for aux variable (filled in _set_noise method)
-        )
+        self.noise_r = None  # Noise for position (filled in _set_noise method)
+        self.noise_Y = None  # Aux variable (filled in _set_noise method)
 
         # Initial conditions
         self.r0 = r0  # Initial position
@@ -512,7 +514,7 @@ class _DiffDiffGenerator(Generator):
 
         if self.r0 is None:
             self.r[0] = np.zeros((self.dim, self.N))  # Default initial positions
-        elif np.shape(self.r0) == (self.dim, self.N) or np.shape(self.r0) == ():
+        elif np.shape(self.r0) == (self.dim, self.N) or np.ndim(self.r0) == 0:
             self.r[0] = self.r0  # User initial positions
         else:
             raise ValueError(
@@ -528,12 +530,13 @@ class _DiffDiffGenerator(Generator):
 
     # Solve coupled Langevin equations
     def _solve(self):
+        sqrt_dt = np.sqrt(self.dt)
         for i in range(self.n - 1):
             # Solving for position
             self.r[i + 1] = self.r[i] + np.sqrt(2 * self.D * self.dt) * self.noise_r[i]
 
             # Solving for auxiliary variable
-            self.Y = self.Y + -self.Y * self.dt + np.sqrt(self.dt) * self.noise_Y[i]
+            self.Y += -self.Y * self.dt + +self.noise_Y[i] * sqrt_dt
 
             # Updating the diffusivities
             self.D = np.sum(self.Y ** 2, axis=0)
@@ -695,6 +698,7 @@ class DiffDiffGenerator(_DiffDiffGenerator):
 
     # Solve dimensionless coupled Langevin equations
     def _solve(self):
+        sqrt_dt = np.sqrt(self.dt)
         for i in range(self.n - 1):
             # Solving for position
             self.r[i + 1] = (
@@ -704,7 +708,7 @@ class DiffDiffGenerator(_DiffDiffGenerator):
             )
 
             # Solving for auxiliary variable
-            self.Y = self.Y + -self.Y * self.dt + np.sqrt(self.dt) * self.noise_Y[i]
+            self.Y += -self.Y * self.dt + self.noise_Y[i] * sqrt_dt
 
             # Updating the diffusivities
             self.D = np.sum(self.Y ** 2, axis=0)
