@@ -1,15 +1,18 @@
 from __future__ import annotations
-import json
+
 import csv
+import json
+import logging
 import os
 from pathlib import Path
 from typing import List, NamedTuple, Tuple, Union
+
 import numpy as np
 
-from yupi.vector import Vector
+import yupi._vel_estimators as vel_estimators
 from yupi.exceptions import LoadTrajectoryError
 from yupi.features import Features
-
+from yupi.vector import Vector
 
 _threshold = 1e-12
 
@@ -36,7 +39,7 @@ class TrajectoryPoint(NamedTuple):
     t: float
 
 
-class Trajectory():
+class Trajectory:
     """
     A Trajectory object represents a multidimensional trajectory.
     It can be iterated to obtain the corresponding point for each
@@ -53,7 +56,7 @@ class Trajectory():
     points : np.ndarray
         Array containing position data as a list of points, by default
         None
-    dimensions : np.ndarray
+    axes : np.ndarray
         Array containing position data as a list of axis, by default
         None
     t : np.ndarray
@@ -71,6 +74,9 @@ class Trajectory():
     lazy : bool
         Defines if the velocity vector is not recalculated every time
         is asked.
+    vel_est : dict
+        Dictionary containing the parameters for the velocity estimation
+        method.
 
     Attributes
     ----------
@@ -97,8 +103,8 @@ class Trajectory():
     Or even create it given all the data for each dimension in a single
     source:
 
-    >>> dims = [[0, 1.2, 3, 2.8], [0, 3.1, 0.7, 1.6]]
-    >>> Trajectory(dimensions=dims)
+    >>> axes = [[0, 1.2, 3, 2.8], [0, 3.1, 0.7, 1.6]]
+    >>> Trajectory(axes=axes)
 
     All of these examples create the same trajectory.
 
@@ -118,20 +124,39 @@ class Trajectory():
         values delta.
     """
 
-    def __init__(self, x: np.ndarray = None, y: np.ndarray = None,
-                 z: np.ndarray = None, points: np.ndarray = None,
-                 dimensions: np.ndarray = None, t: np.ndarray = None,
-                 ang: np.ndarray = None, dt: float = None, t0: float = 0.0,
-                 traj_id: str = None, lazy: bool = False):
+    __vel_est = {
+        "method": vel_estimators.VelocityMethod.LINEAR_DIFF,
+        "window_type": vel_estimators.WindowType.CENTRAL,
+    }
 
+    def __init__(
+        self,
+        x: np.ndarray = None,
+        y: np.ndarray = None,
+        z: np.ndarray = None,
+        points: np.ndarray = None,
+        axes: np.ndarray = None,
+        t: np.ndarray = None,
+        ang: np.ndarray = None,
+        dt: float = None,
+        t0: float = 0.0,
+        traj_id: str = None,
+        lazy: bool = False,
+        vel_est: dict = None,
+    ):
+
+        # Position data validation
         from_xyz = x is not None
         from_points = points is not None
-        from_dimensions = dimensions is not None
+        from_axes = axes is not None
 
-        if from_xyz + from_points + from_dimensions > 1:
-            raise ValueError("Positional data must come only from one way: "
-                             "'xyz' data, 'points' data or 'dimensions' data.")
+        if from_xyz + from_points + from_axes > 1:
+            raise ValueError(
+                "Positional data must come only from one way: "
+                "'xyz' data, 'points' data or 'axes' data."
+            )
 
+        # Set position data
         self.r = None
         data: List[Vector] = [t, ang]
         lengths = [len(item) for item in data if item is not None]
@@ -141,53 +166,121 @@ class Trajectory():
                 data[i] = Vector.create(item, dtype=float)
 
         if from_xyz:
-            dimensions = [d for d in [x, y, z] if d is not None]
-            from_dimensions = True
+            axes = [d for d in [x, y, z] if d is not None]
+            from_axes = True
 
-        if from_dimensions and len(dimensions) > 0:
-            lengths.extend([len(d) for d in dimensions])
-            self.r = Vector.create(dimensions, dtype=float).T
+        if from_axes and len(axes) > 0:
+            lengths.extend([len(d) for d in axes])
+            self.r = Vector.create(axes, dtype=float).T
 
         if from_points:
             lengths.append(len(points))
             self.r = Vector.create(points, dtype=float)
 
         if self.r is None:
-            raise ValueError('No position data were given.')
+            raise ValueError("No position data were given.")
 
         if lengths.count(lengths[0]) != len(lengths):
-            raise ValueError('All input arrays must have the same shape.')
+            raise ValueError("All input arrays must have the same shape.")
+
+        if len(self.r) < 2:
+            raise ValueError("The trajectory must contain at least 2 points.")
 
         self.__dt = dt
         self.dt_mean = dt
         self.__t0 = t0
         self.__t = data[0]
         self.ang = data[1]
-        self.traj_id= traj_id
+        self.traj_id = traj_id
         self.lazy = lazy
 
+        # Set time data
         if self.__t is None:
             self.dt_mean = dt if dt is not None else 1.0
             self.dt_std = 0
-            self.__v: Vector = self.r.delta / self.dt
         else:
             self.dt_mean = np.mean(np.array(self.__t.delta))
             self.dt_std = np.std(np.array(self.__t.delta))
-            self.__v: Vector = (self.r.delta.T / self.__t.delta).T
 
+        # Velocity estimation
+        self.vel_est = Trajectory.__vel_est.copy()
+        if vel_est is not None:
+            self.vel_est.update(vel_est)
+
+        self.recalculate_velocity()
+
+        # Time parameters validation
         if t is not None and dt is not None:
             if abs(self.dt_mean - dt) > _threshold:
-                raise ValueError("You are giving 'dt' and 't' but 'dt' "
-                                "does not match with time values delta.")
+                raise ValueError(
+                    "You are giving 'dt' and 't' but 'dt' "
+                    "does not match with time values delta."
+                )
             if abs(self.dt_std - 0) > _threshold:
-                raise ValueError("You are giving 'dt' and 't' but 't' is "
-                                    "not uniformly spaced.")
+                raise ValueError(
+                    "You are giving 'dt' and 't' but 't' is " "not uniformly spaced."
+                )
             if abs(self.__t[0] - t0) > _threshold:
-                raise ValueError("You are giving 'dt' and 't' but 't0' is not "
-                                 "the same as the first value of 't'.")
+                raise ValueError(
+                    "You are giving 'dt' and 't' but 't0' is not "
+                    "the same as the first value of 't'."
+                )
 
         self.features = Features(self)
 
+    def set_vel_method(
+        self,
+        method: vel_estimators.VelocityMethod,
+        window_type: vel_estimators.WindowType = vel_estimators.WindowType.CENTRAL,
+        accuracy: int = 1,
+    ):
+        """
+        Set the method to calculate the velocity.
+
+        Parameters
+        ----------
+        method : VelocityMethod
+            Method to calculate the velocity.
+        window_type : WindowType
+            Type of window to use to calculate the velocity. By default,
+            the central window is used.
+        accuracy : int
+            Accuracy of the velocity estimation (only valid for
+            FORNBERG_DIFF method). By default, the accuracy is 1.
+        """
+        self.vel_est = {
+            "method": method,
+            "window_type": window_type,
+            "accuracy": accuracy,
+        }
+        self.recalculate_velocity()
+
+    @staticmethod
+    def global_vel_method(
+        method: vel_estimators.VelocityMethod,
+        window_type: vel_estimators.WindowType = vel_estimators.WindowType.CENTRAL,
+        accuracy: int = 1,
+    ):
+        """
+        Set the method to calculate the velocity.
+
+        Parameters
+        ----------
+        method : VelocityMethod
+            Method to calculate the velocity.
+        window_type : WindowType
+            Type of window to use to calculate the velocity. By default,
+            the central window is used.
+        accuracy : int
+            Accuracy of the velocity estimation (only valid for
+            FORNBERG method). By default, the accuracy is 1.
+        """
+        Trajectory.__vel_est = {
+            "method": method,
+            "window_type": window_type,
+            "accuracy": accuracy,
+        }
+
     @property
     def dt(self) -> float:
         """
@@ -222,7 +315,7 @@ class Trajectory():
                 data[1] = self.ang[index]
 
             # Velocity
-            data[2] = self.v[index - 1] if index > 0 else Vector.create([0]*self.dim)
+            data[2] = self.v[index - 1] if index > 0 else Vector.create([0] * self.dim)
 
             # Time
             if self.__t is not None:
@@ -243,10 +336,17 @@ class Trajectory():
             if self.uniformly_spaced:
                 new_dt = self.dt * step
                 new_t0 = self.__t0 + start * self.dt
-                return Trajectory(points=new_points, ang=new_ang, dt=new_dt,
-                                  t0=new_t0)
+                return Trajectory(
+                    points=new_points,
+                    ang=new_ang,
+                    dt=new_dt,
+                    t0=new_t0,
+                    vel_est=self.vel_est,
+                )
             new_t = self.t[start:stop:step]
-            return Trajectory(points=new_points, ang=new_ang, t=new_t)
+            return Trajectory(
+                points=new_points, ang=new_ang, t=new_t, vel_est=self.vel_est
+            )
 
     def __iter__(self):
         for i in range(len(self)):
@@ -299,15 +399,27 @@ class Trajectory():
             Velocity vector.
         """
 
-        if self.uniformly_spaced:
-            self.__v: Vector = self.r.delta / self.dt
-        else:
-            self.__v: Vector = (self.r.delta.T / self.__t.delta).T
+        self.__v = vel_estimators.estimate_velocity(self, **self.vel_est)
         return self.__v
 
     @property
     def v(self) -> Vector:
         """Vector : Velocity vector"""
+        if self.__v is None:
+            method = self.vel_est["method"]
+            win_type = self.vel_est.get(
+                "window_type", vel_estimators.WindowType.CENTRAL
+            )
+            acc = self.vel_est.get("accuracy", 1)
+            acc_text = (
+                f" and accuracy {acc}"
+                if method == vel_estimators.VelocityMethod.FORNBERG_DIFF
+                else ""
+            )
+            raise ValueError(
+                f"Trajectory velocity can not be estimated using {method.name} "
+                f"method with window type {win_type.name}{acc_text}."
+            )
         if self.lazy:
             return self.__v
         return self.recalculate_velocity()
@@ -316,7 +428,7 @@ class Trajectory():
     def t(self) -> Vector:
         """Vector : Time vector"""
         if self.__t is None:
-            dt_vec = [self.__t0 + self.dt*i for i in range(len(self))]
+            dt_vec = [self.__t0 + self.dt * i for i in range(len(self))]
             self.__t = Vector.create(dt_vec)
         return self.__t
 
@@ -346,8 +458,9 @@ class Trajectory():
         """
 
         if self.dim != 2:
-            raise TypeError('Polar offsets can only be applied on 2 '
-                            'dimensional trajectories')
+            raise TypeError(
+                "Polar offsets can only be applied on 2 " "dimensional trajectories"
+            )
 
         # From cartesian to polar
         x, y = self.r.x, self.r.y
@@ -360,7 +473,7 @@ class Trajectory():
         # From polar to cartesian
         x = rad * np.cos(ang)
         y = rad * np.sin(ang)
-        self.r = Vector.create([x,y]).T
+        self.r = Vector.create([x, y]).T
 
     def rotate2d(self, angle: float):
         """
@@ -391,20 +504,37 @@ class Trajectory():
         """
 
         if self.dim != 3:
-            raise TypeError('3D rotations can only be applied on 3 '
-                            'dimensional trajectories')
+            raise TypeError(
+                "3D rotations can only be applied on 3 " "dimensional trajectories"
+            )
 
         vec: Vector = Vector.create(vector)
         if len(vec) != 3:
-            raise ValueError('The vector must have 3 components')
+            raise ValueError("The vector must have 3 components")
 
         vec = vec / vec.norm
         vx, vy, vz = vec[0], vec[1], vec[2]
         c, s = np.cos(angle), np.sin(angle)
 
-        R = np.array([[vx*vx*(1-c)+c, vx*vy*(1-c)-vz*s, vx*vz*(1-c)+vy*s],
-                      [vx*vy*(1-c)+vz*s, vy*vy*(1-c)+c, vy*vz*(1-c)-vx*s],
-                      [vx*vz*(1-c)-vy*s, vy*vz*(1-c)+vx*s, vz*vz*(1-c)+c]])
+        R = np.array(
+            [
+                [
+                    vx * vx * (1 - c) + c,
+                    vx * vy * (1 - c) - vz * s,
+                    vx * vz * (1 - c) + vy * s,
+                ],
+                [
+                    vx * vy * (1 - c) + vz * s,
+                    vy * vy * (1 - c) + c,
+                    vy * vz * (1 - c) - vx * s,
+                ],
+                [
+                    vx * vz * (1 - c) - vy * s,
+                    vy * vz * (1 - c) + vx * s,
+                    vz * vz * (1 - c) + c,
+                ],
+            ]
+        )
 
         self.r = Vector.create(np.dot(self.r, R))
 
@@ -418,8 +548,14 @@ class Trajectory():
             Copy of the trajectory.
         """
 
-        return Trajectory(points=self.r, t=self.__t, ang=self.ang, dt=self.dt,
-                          lazy=self.lazy)
+        return Trajectory(
+            points=self.r,
+            t=self.__t,
+            ang=self.ang,
+            dt=self.__dt,
+            lazy=self.lazy,
+            vel_est=self.vel_est,
+        )
 
     def _operable_with(self, other: Trajectory, threshold=None) -> bool:
         if threshold is None:
@@ -442,19 +578,22 @@ class Trajectory():
         if isinstance(other, (list, tuple, np.ndarray)):
             offset = np.array(other, dtype=float)
             if len(offset) != self.dim:
-                raise ValueError('Offset must be the same shape as the other '
-                                 'trajectory points')
+                raise ValueError(
+                    "Offset must be the same shape as the other " "trajectory points"
+                )
             self.r += offset
             return self
 
         if isinstance(other, Trajectory):
             if not self._operable_with(other):
-                raise ValueError('Incompatible trajectories')
+                raise ValueError("Incompatible trajectories")
             self.r += other.r
             return self
 
-        raise TypeError("unsoported operation (+) between 'Trajectory' and "
-                        f"'{type(other).__name__}'")
+        raise TypeError(
+            "unsoported operation (+) between 'Trajectory' and "
+            f"'{type(other).__name__}'"
+        )
 
     def __isub__(self, other):
         if isinstance(other, (int, float)):
@@ -464,19 +603,22 @@ class Trajectory():
         if isinstance(other, (list, tuple, np.ndarray)):
             offset = np.array(other, dtype=float)
             if len(offset) != self.dim:
-                raise ValueError('Offset must be the same shape as the other '
-                                 'trajectory points')
+                raise ValueError(
+                    "Offset must be the same shape as the other " "trajectory points"
+                )
             self.r -= offset
             return self
 
         if isinstance(other, Trajectory):
             if not self._operable_with(other):
-                raise ValueError('Incompatible trajectories')
+                raise ValueError("Incompatible trajectories")
             self.r -= other.r
             return self
 
-        raise TypeError("unsoported operation (-) between 'Trajectory' and "
-                        f"'{type(other).__name__}'")
+        raise TypeError(
+            "unsoported operation (-) between 'Trajectory' and "
+            f"'{type(other).__name__}'"
+        )
 
     def __add__(self, other):
         traj = self.copy()
@@ -499,8 +641,10 @@ class Trajectory():
             self.r *= other
             return self
 
-        raise TypeError("unsoported operation (*) between 'Trajectory' and "
-                        f"'{type(other).__name__}'")
+        raise TypeError(
+            "unsoported operation (*) between 'Trajectory' and "
+            f"'{type(other).__name__}'"
+        )
 
     def __mul__(self, other):
         traj = self.copy()
@@ -510,8 +654,9 @@ class Trajectory():
     def __rmul__(self, other):
         return self * other
 
-    def turning_angles(self, accumulate=False, degrees=False,
-                       centered=False, wrap=True):
+    def turning_angles(
+        self, accumulate=False, degrees=False, centered=False, wrap=True
+    ):
         """
         Return the sequence of turning angles that forms the trajectory.
 
@@ -550,7 +695,7 @@ class Trajectory():
         if not accumulate:
             theta = np.ediff1d(theta)  # Relative turning angles
         else:
-            theta -= theta[0]          # Accumulative turning angles
+            theta -= theta[0]  # Accumulative turning angles
 
         if degrees:
             theta = np.rad2deg(theta)
@@ -575,27 +720,53 @@ class Trajectory():
             return {d: list(v) for d, v in enumerate(vec)}
 
         ang = None if self.ang is None else self.ang.T
-        json_dict = {
-            'id': self.traj_id,
-            'dt': self.__dt,
-            'r': convert_to_list(self.r.T),
-            'ang': convert_to_list(ang),
-            't': convert_to_list(self.__t)
+
+        default_vel_est_method = vel_estimators.VelocityMethod.LINEAR_DIFF
+        default_vel_est_window = vel_estimators.WindowType.CENTRAL
+        default_vel_est_accuracy = 1
+        vel_est = {
+            "method": self.vel_est.get("method", default_vel_est_method).value,
+            "window_type": self.vel_est.get("window", default_vel_est_window).value,
+            "accuracy": self.vel_est.get("accuracy", default_vel_est_accuracy),
         }
-        with open(str(path), 'w') as traj_file:
+
+        json_dict = {
+            "id": self.traj_id,
+            "dt": self.__dt,
+            "r": convert_to_list(self.r.T),
+            "ang": convert_to_list(ang),
+            "t": convert_to_list(self.__t),
+            "vel_est": vel_est,
+        }
+        with open(str(path), "w") as traj_file:
             json.dump(json_dict, traj_file)
 
     def _save_csv(self, path):
-        with open(str(path), 'w', newline='') as traj_file:
-            writer = csv.writer(traj_file, delimiter=',')
+        with open(str(path), "w", newline="") as traj_file:
+            writer = csv.writer(traj_file, delimiter=",")
             ang_shape = 0 if self.ang is None else self.ang.shape[1]
             writer.writerow([self.traj_id, self.__dt, self.dim, ang_shape])
+
+            default_vel_est_method = vel_estimators.VelocityMethod.LINEAR_DIFF
+            default_vel_est_window = vel_estimators.WindowType.CENTRAL
+            default_vel_est_accuracy = 1
+            method = self.vel_est.get("method", default_vel_est_method).value
+            window = self.vel_est.get("window", default_vel_est_window).value
+            accuracy = self.vel_est.get("accuracy", default_vel_est_accuracy)
+            print(method, window, accuracy)
+            writer.writerow([method, window, accuracy])
+
             for tp in self:
                 row = np.hstack(np.array([tp.r, tp.ang, tp.t], dtype=object))
                 writer.writerow(row)
 
-    def save(self, file_name: str, path: str = '.', file_type: str = 'json',
-             overwrite: bool = True):
+    def save(
+        self,
+        file_name: str,
+        path: str = ".",
+        file_type: str = "json",
+        overwrite: bool = True,
+    ):
         """
         Saves the trajectory to disk.
 
@@ -628,22 +799,26 @@ class Trajectory():
         """
 
         # Build full path
-        full_path = Path(path) / Path(f'{file_name}.{file_type}')
+        full_path = Path(path) / Path(f"{file_name}.{file_type}")
 
         # Check file existance
         if not overwrite and full_path.exists():
             raise ValueError(f"File '{str(full_path)}' already exist")
 
-        if file_type == 'json':
+        if file_type == "json":
             self._save_json(full_path)
-        elif file_type == 'csv':
+        elif file_type == "csv":
             self._save_csv(full_path)
         else:
             raise ValueError(f"Invalid export file type '{file_type}'")
 
     @staticmethod
-    def save_trajectories(trajectories: list, folder_path: str = '.',
-                          file_type: str = 'json', overwrite: bool = True):
+    def save_trajectories(
+        trajectories: list,
+        folder_path: str = ".",
+        file_type: str = "json",
+        overwrite: bool = True,
+    ):
         """
         Saves a list of trajectories to disk. Each Trajectory object
         will be saved in a separate file inside the given folder.
@@ -671,34 +846,43 @@ class Trajectory():
 
         for i, traj in enumerate(trajectories):
             path = str(Path(folder_path))
-            name = str(Path(f'trajectory_{i}'))
+            name = str(Path(f"trajectory_{i}"))
             traj.save(name, path, file_type, overwrite)
 
     @staticmethod
     def _load_json(path: str):
-        with open(path, 'r') as traj_file:
+        with open(path, "r") as traj_file:
             data = json.load(traj_file)
 
-            traj_id = data['id']
-            dt = data['dt']
-            t = data['t']
+            traj_id = data["id"]
+            dt = data["dt"]
+            t = data["t"]
             ang = None
 
-            if data['ang'] is not None:
-                ang_values = list(data['ang'].values())
+            if data["ang"] is not None:
+                ang_values = list(data["ang"].values())
                 ang = Vector.create(ang_values).T
 
-            dims = list(data['r'].values())
+            axes = list(data["r"].values())
+            vel_est = data.get("vel_est", None)
+            if vel_est is None:
+                vel_est = Trajectory.__vel_est
+            else:
+                vel_est["method"] = vel_estimators.VelocityMethod(vel_est["method"])
+                vel_est["window_type"] = vel_estimators.WindowType(
+                    vel_est["window_type"]
+                )
 
-            return Trajectory(dimensions=dims, t=t, ang=ang, dt=dt,
-                              traj_id=traj_id)
+            return Trajectory(
+                axes=axes, t=t, ang=ang, dt=dt, traj_id=traj_id, vel_est=vel_est
+            )
 
     @staticmethod
     def _load_csv(path: str):
-        with open(path, 'r') as traj_file:
+        with open(path, "r") as traj_file:
 
             def check_empty_val(val, cast=True):
-                if val == '':
+                if val == "":
                     return None
                 return float(val) if cast else val
 
@@ -715,6 +899,14 @@ class Trajectory():
                     ang = [[] for _ in range(ang_dim)]
                     continue
 
+                if i == 1:
+                    vel_est = {
+                        "method": vel_estimators.VelocityMethod(int(row[0])),
+                        "window_type": vel_estimators.WindowType(int(row[1])),
+                        "accuracy": int(row[2]),
+                    }
+                    continue
+
                 for j in range(dim):
                     r[j].append(float(row[j]))
 
@@ -726,8 +918,7 @@ class Trajectory():
             if not ang:
                 ang = None
 
-            return Trajectory(dimensions=r, t=t, ang=ang, dt=dt,
-                              traj_id=traj_id)
+            return Trajectory(axes=r, t=t, ang=ang, dt=dt, traj_id=traj_id)
 
     @staticmethod
     def load(file_path: str):
@@ -758,16 +949,16 @@ class Trajectory():
 
         # Check valid path
         if not path.exists():
-            raise ValueError('Path does not exist.')
+            raise ValueError("Path does not exist.")
         if not path.is_file():
             raise ValueError("Path must be a file.")
 
         file_type = path.suffix
 
         try:
-            if file_type == '.json':
+            if file_type == ".json":
                 return Trajectory._load_json(file_path)
-            elif file_type == '.csv':
+            elif file_type == ".csv":
                 return Trajectory._load_csv(file_path)
             else:
                 raise ValueError("Invalid file type.")
@@ -775,7 +966,7 @@ class Trajectory():
             raise LoadTrajectoryError(path) from exc
 
     @staticmethod
-    def load_folder(folder_path='.', recursively: bool = False):
+    def load_folder(folder_path=".", recursively: bool = False):
         """
         Loads all the trajectories from a folder.
 
@@ -800,7 +991,7 @@ class Trajectory():
                 try:
                     trajectories.append(Trajectory.load(path))
                 except LoadTrajectoryError as load_exception:
-                    print(f'Ignoring: {load_exception.path}')
+                    print(f"Ignoring: {load_exception.path}")
             if not recursively:
                 break
         return trajectories
