@@ -2,8 +2,7 @@ import numpy as np
 from yupi.trajectory import Trajectory
 
 
-def exp_convolutional_filter(traj: Trajectory, gamma: float,
-                             new_traj_id: str = None):
+def exp_convolutional_filter(traj: Trajectory, gamma: float, new_traj_id: str = None):
     """
     Returns a smoothed version of the trajectory `traj`
     by taking a weighted average over past values.
@@ -28,5 +27,7 @@ def exp_convolutional_filter(traj: Trajectory, gamma: float,
     for i in range(len(traj) - 1):
         R[i + 1] = R[i] - gamma * (R[i] - r[i]) * dt[i]
 
-    smooth_traj = Trajectory(points=R, t=traj.t, traj_id=new_traj_id)
+    smooth_traj = Trajectory(
+        points=R, t=traj.t, traj_id=new_traj_id, vel_est=traj.vel_est
+    )
     return smooth_traj
