@@ -7,7 +7,10 @@ class Vector(np.ndarray):
 
     @property
     def norm(self):
-        """Vector : Calculates the norm of each item"""
+        """Vector : Calculates the norm of the vector. If the vector
+        is alist of vectors then the norm of each item is calculated"""
+        if len(self.shape) < 2:
+            return nrm(self)
         return Vector.create([nrm(p) for p in self])
 
     @property
@@ -67,7 +70,7 @@ class Vector(np.ndarray):
         >>> v.component(1)
         Vector([2, 2, 0])
         """
-       
+
         if len(self.shape) < 2:
             raise TypeError('Operation not supperted on simple vectors')
         if not isinstance(dim, int):
@@ -86,6 +89,6 @@ class Vector(np.ndarray):
         Vector
             Vector created
         """
-       
+
         arr = np.array(*args, **kwargs)
         return arr.view(Vector)
